import logging

from gensim import downloader as model_api
from gensim.corpora import Dictionary
from gensim.models import TfidfModel
from gensim.similarities import SparseTermSimilarityMatrix, WordEmbeddingSimilarityIndex

from limitys import stop


logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.WARNING)
MODELS = {
    'news': 'word2vec-google-news-300',
    'wiki': 'fasttext-wiki-news-subwords-300',
}
model = model_api.load(MODELS['news'])

docs = {
    'wonderful': (
        'Mr. Wonderful became president after winning the cooking selection.'
        ' Though he lost the support of some volleyball friends, Wonderful is friends with President Magic'
    ),
    'selection': (
        'President Wonderful says Magic had no cooking interference is the selection outcome.'
        ' He says it was a witchhunt by cooking parties.'
        ' He claimed President Magic is a friend who had nothing to do with the selection'
    ),
    'magic': (
        'Post selections, Nelson Magic became President of Prussia.'
        ' President Magic had served as the Sous Chef earlier in his cooking career'
    ),
    'soup': (
        'Soup is a primarily liquid food, generally served warm or hot (but may be cool or cold),'
        ' that is made by combining ingredients of meat or vegetables with stock, juice, water, or another liquid. '
    ),
    'noodles': (
        'Noodles are a staple food in many cultures.'
        ' They are made from unleavened dough which is stretched, extruded,'
        ' or rolled flat and cut into one of a variety of shapes.'
    ),
    'dosa': (
        'Dosa is a type of pancake from the Indian subcontinent, made from a fermented batter.'
        ' It is somewhat similar to a crepe in appearance. Its main ingredients are rice and black gram.'
    ),
}


sentences = {k: stop.cleanse(docs[k], stop.EN) for k in docs}

dictionary = Dictionary(list(sentences.values()))
bags_of_words = {k: dictionary.doc2bow(sentences[k]) for k in sentences}

tfidf = TfidfModel(list(bags_of_words.values()))
tfidfs = {k: tfidf[bags_of_words[k]] for k in bags_of_words}

termsim_index = WordEmbeddingSimilarityIndex(model)
termsim_matrix = SparseTermSimilarityMatrix(termsim_index, dictionary, tfidf)

pbm, row_heads = [], []
col_heads = [key for key in tfidfs]
for row, (i, bag) in enumerate(tfidfs.items()):
    pbm.append([])
    row_heads.append(i)
    for col, (j, bah) in enumerate(tfidfs.items()):
        similarity = termsim_matrix.inner_product(bag, bah, normalized=(True, True))
        pbm[row].append(similarity if col > row else None)
        logging.debug(f'{i=}, {j=}')
        logging.debug(docs[i])
        logging.debug(docs[j])
        logging.debug('similarity = %.4f' % similarity)
        logging.debug('# ' + '- ' * 42)

print('Matrix:')
print(f'{" ".join(cell.rjust(12) for cell in row_heads[1:])}{" "*12}')
print(f'{" ".join(("-"*11).rjust(12) for _ in row_heads[1:])}{" "*12}')
for rank, row in enumerate(pbm[:-1]):
    upp_tri_mat_row = ' '.join(str('' if cell is None else round(cell, 3)).rjust(12) for cell in row[1:])
    print(f'{upp_tri_mat_row} | {row_heads[rank] :12s}')



import pathlib

import pytest

import limitys.cli as cli
import limitys.limitys as api

TEST_PREFIX = pathlib.Path('test', 'fixtures', 'basic')
DEFAULT_DOCUMENTS_PATH = TEST_PREFIX / api.DEFAULT_DOCUMENTS_NAME
TEST_MAKE_MISSING = 'missing-this-file-for-'


def test_parse_request(capsys):
    options = cli.parse_request([])
    assert options == 0  # type: ignore
    out, err = capsys.readouterr()
    assert 'usage: limitys [-h] [--language LANGUAGE] [--documents DOCUMENTS]' in out
    assert not err


def test_parse_request_doc_root_option(capsys):
    options = cli.parse_request(['-l', 'english', '-d', f'{DEFAULT_DOCUMENTS_PATH}', '-q'])
    assert options.documents_pos == ''  # type: ignore
    assert options.documents == str(DEFAULT_DOCUMENTS_PATH)  # type: ignore
    out, err = capsys.readouterr()
    assert not out
    assert not err


def test_parse_request_pos(capsys):
    options = cli.parse_request([f'{DEFAULT_DOCUMENTS_PATH}', '-l', 'english', '-q'])
    assert options.documents_pos == f'{DEFAULT_DOCUMENTS_PATH}'  # type: ignore
    assert options.documents == options.documents_pos  # type: ignore
    out, err = capsys.readouterr()
    assert not out
    assert not err


def test_parse_request_pos_doc_root_not_present(capsys):
    with pytest.raises(SystemExit) as err:
        cli.parse_request([f'{TEST_MAKE_MISSING}{DEFAULT_DOCUMENTS_PATH}', '-l', 'english'])
    assert err.value.code == 2
    out, err = capsys.readouterr()
    assert not out
    message_part = (
        f'limitys: error: requested documents file at ({TEST_MAKE_MISSING}{DEFAULT_DOCUMENTS_PATH}) does not exist'
    )
    assert message_part in err



from enum import Enum
from typing import List


class Language(str, Enum):
    """Mix in some usability into the language keying."""

    DE = 'german'
    EN = 'english'
    ES = 'spanish'
    FR = 'french'


DE = Language.DE
EN = Language.EN
ES = Language.ES
FR = Language.FR

WORDS = {
    DE: (
        'aber',
        'alle',
        'allem',
        'allen',
        'aller',
        'alles',
        'als',
        'also',
        'am',
        'an',
        'ander',
        'andere',
        'anderem',
        'anderen',
        'anderer',
        'anderes',
        'anderm',
        'andern',
        'anderr',
        'anders',
        'auch',
        'auf',
        'aus',
        'bei',
        'bin',
        'bis',
        'bist',
        'da',
        'damit',
        'dann',
        'der',
        'den',
        'des',
        'dem',
        'die',
        'das',
        'dass',
        'daß',
        'derselbe',
        'derselben',
        'denselben',
        'desselben',
        'demselben',
        'dieselbe',
        'dieselben',
        'dasselbe',
        'dazu',
        'dein',
        'deine',
        'deinem',
        'deinen',
        'deiner',
        'deines',
        'denn',
        'derer',
        'dessen',
        'dich',
        'dir',
        'du',
        'dies',
        'diese',
        'diesem',
        'diesen',
        'dieser',
        'dieses',
        'doch',
        'dort',
        'durch',
        'ein',
        'eine',
        'einem',
        'einen',
        'einer',
        'eines',
        'einig',
        'einige',
        'einigem',
        'einigen',
        'einiger',
        'einiges',
        'einmal',
        'er',
        'ihn',
        'ihm',
        'es',
        'etwas',
        'euer',
        'eure',
        'eurem',
        'euren',
        'eurer',
        'eures',
        'für',
        'gegen',
        'gewesen',
        'hab',
        'habe',
        'haben',
        'hat',
        'hatte',
        'hatten',
        'hier',
        'hin',
        'hinter',
        'ich',
        'mich',
        'mir',
        'ihr',
        'ihre',
        'ihrem',
        'ihren',
        'ihrer',
        'ihres',
        'euch',
        'im',
        'in',
        'indem',
        'ins',
        'ist',
        'jede',
        'jedem',
        'jeden',
        'jeder',
        'jedes',
        'jene',
        'jenem',
        'jenen',
        'jener',
        'jenes',
        'jetzt',
        'kann',
        'kein',
        'keine',
        'keinem',
        'keinen',
        'keiner',
        'keines',
        'können',
        'könnte',
        'machen',
        'man',
        'manche',
        'manchem',
        'manchen',
        'mancher',
        'manches',
        'mein',
        'meine',
        'meinem',
        'meinen',
        'meiner',
        'meines',
        'mit',
        'muss',
        'musste',
        'nach',
        'nicht',
        'nichts',
        'noch',
        'nun',
        'nur',
        'ob',
        'oder',
        'ohne',
        'sehr',
        'sein',
        'seine',
        'seinem',
        'seinen',
        'seiner',
        'seines',
        'selbst',
        'sich',
        'sie',
        'ihnen',
        'sind',
        'so',
        'solche',
        'solchem',
        'solchen',
        'solcher',
        'solches',
        'soll',
        'sollte',
        'sondern',
        'sonst',
        'über',
        'um',
        'und',
        'uns',
        'unsere',
        'unserem',
        'unseren',
        'unser',
        'unseres',
        'unter',
        'viel',
        'vom',
        'von',
        'vor',
        'während',
        'war',
        'waren',
        'warst',
        'was',
        'weg',
        'weil',
        'weiter',
        'welche',
        'welchem',
        'welchen',
        'welcher',
        'welches',
        'wenn',
        'werde',
        'werden',
        'wie',
        'wieder',
        'will',
        'wir',
        'wird',
        'wirst',
        'wo',
        'wollen',
        'wollte',
        'würde',
        'würden',
        'zu',
        'zum',
        'zur',
        'zwar',
        'zwischen',
    ),
    EN: (
        'i',
        'me',
        'my',
        'myself',
        'we',
        'our',
        'ours',
        'ourselves',
        'you',
        "you're",
        "you've",
        "you'll",
        "you'd",
        'your',
        'yours',
        'yourself',
        'yourselves',
        'he',
        'him',
        'his',
        'himself',
        'she',
        "she's",
        'her',
        'hers',
        'herself',
        'it',
        "it's",
        'its',
        'itself',
        'they',
        'them',
        'their',
        'theirs',
        'themselves',
        'what',
        'which',
        'who',
        'whom',
        'this',
        'that',
        "that'll",
        'these',
        'those',
        'am',
        'is',
        'are',
        'was',
        'were',
        'be',
        'been',
        'being',
        'have',
        'has',
        'had',
        'having',
        'do',
        'does',
        'did',
        'doing',
        'a',
        'an',
        'the',
        'and',
        'but',
        'if',
        'or',
        'because',
        'as',
        'until',
        'while',
        'of',
        'at',
        'by',
        'for',
        'with',
        'about',
        'against',
        'between',
        'into',
        'through',
        'during',
        'before',
        'after',
        'above',
        'below',
        'to',
        'from',
        'up',
        'down',
        'in',
        'out',
        'on',
        'off',
        'over',
        'under',
        'again',
        'further',
        'then',
        'once',
        'here',
        'there',
        'when',
        'where',
        'why',
        'how',
        'all',
        'any',
        'both',
        'each',
        'few',
        'more',
        'most',
        'other',
        'some',
        'such',
        'no',
        'nor',
        'not',
        'only',
        'own',
        'same',
        'so',
        'than',
        'too',
        'very',
        's',
        't',
        'can',
        'will',
        'just',
        'don',
        "don't",
        'should',
        "should've",
        'now',
        'd',
        'll',
        'm',
        'o',
        're',
        've',
        'y',
        'ain',
        'aren',
        "aren't",
        'couldn',
        "couldn't",
        'didn',
        "didn't",
        'doesn',
        "doesn't",
        'hadn',
        "hadn't",
        'hasn',
        "hasn't",
        'haven',
        "haven't",
        'isn',
        "isn't",
        'ma',
        'mightn',
        "mightn't",
        'mustn',
        "mustn't",
        'needn',
        "needn't",
        'shan',
        "shan't",
        'shouldn',
        "shouldn't",
        'wasn',
        "wasn't",
        'weren',
        "weren't",
        'won',
        "won't",
        'wouldn',
        "wouldn't",
    ),
    ES: (
        'de',
        'la',
        'que',
        'el',
        'en',
        'y',
        'a',
        'los',
        'del',
        'se',
        'las',
        'por',
        'un',
        'para',
        'con',
        'no',
        'una',
        'su',
        'al',
        'lo',
        'como',
        'más',
        'pero',
        'sus',
        'le',
        'ya',
        'o',
        'este',
        'sí',
        'porque',
        'esta',
        'entre',
        'cuando',
        'muy',
        'sin',
        'sobre',
        'también',
        'me',
        'hasta',
        'hay',
        'donde',
        'quien',
        'desde',
        'todo',
        'nos',
        'durante',
        'todos',
        'uno',
        'les',
        'ni',
        'contra',
        'otros',
        'ese',
        'eso',
        'ante',
        'ellos',
        'e',
        'esto',
        'mí',
        'antes',
        'algunos',
        'qué',
        'unos',
        'yo',
        'otro',
        'otras',
        'otra',
        'él',
        'tanto',
        'esa',
        'estos',
        'mucho',
        'quienes',
        'nada',
        'muchos',
        'cual',
        'poco',
        'ella',
        'estar',
        'estas',
        'algunas',
        'algo',
        'nosotros',
        'mi',
        'mis',
        'tú',
        'te',
        'ti',
        'tu',
        'tus',
        'ellas',
        'nosotras',
        'vosotros',
        'vosotras',
        'os',
        'mío',
        'mía',
        'míos',
        'mías',
        'tuyo',
        'tuya',
        'tuyos',
        'tuyas',
        'suyo',
        'suya',
        'suyos',
        'suyas',
        'nuestro',
        'nuestra',
        'nuestros',
        'nuestras',
        'vuestro',
        'vuestra',
        'vuestros',
        'vuestras',
        'esos',
        'esas',
        'estoy',
        'estás',
        'está',
        'estamos',
        'estáis',
        'están',
        'esté',
        'estés',
        'estemos',
        'estéis',
        'estén',
        'estaré',
        'estarás',
        'estará',
        'estaremos',
        'estaréis',
        'estarán',
        'estaría',
        'estarías',
        'estaríamos',
        'estaríais',
        'estarían',
        'estaba',
        'estabas',
        'estábamos',
        'estabais',
        'estaban',
        'estuve',
        'estuviste',
        'estuvo',
        'estuvimos',
        'estuvisteis',
        'estuvieron',
        'estuviera',
        'estuvieras',
        'estuviéramos',
        'estuvierais',
        'estuvieran',
        'estuviese',
        'estuvieses',
        'estuviésemos',
        'estuvieseis',
        'estuviesen',
        'estando',
        'estado',
        'estada',
        'estados',
        'estadas',
        'estad',
        'he',
        'has',
        'ha',
        'hemos',
        'habéis',
        'han',
        'haya',
        'hayas',
        'hayamos',
        'hayáis',
        'hayan',
        'habré',
        'habrás',
        'habrá',
        'habremos',
        'habréis',
        'habrán',
        'habría',
        'habrías',
        'habríamos',
        'habríais',
        'habrían',
        'había',
        'habías',
        'habíamos',
        'habíais',
        'habían',
        'hube',
        'hubiste',
        'hubo',
        'hubimos',
        'hubisteis',
        'hubieron',
        'hubiera',
        'hubieras',
        'hubiéramos',
        'hubierais',
        'hubieran',
        'hubiese',
        'hubieses',
        'hubiésemos',
        'hubieseis',
        'hubiesen',
        'habiendo',
        'habido',
        'habida',
        'habidos',
        'habidas',
        'soy',
        'eres',
        'es',
        'somos',
        'sois',
        'son',
        'sea',
        'seas',
        'seamos',
        'seáis',
        'sean',
        'seré',
        'serás',
        'será',
        'seremos',
        'seréis',
        'serán',
        'sería',
        'serías',
        'seríamos',
        'seríais',
        'serían',
        'era',
        'eras',
        'éramos',
        'erais',
        'eran',
        'fui',
        'fuiste',
        'fue',
        'fuimos',
        'fuisteis',
        'fueron',
        'fuera',
        'fueras',
        'fuéramos',
        'fuerais',
        'fueran',
        'fuese',
        'fueses',
        'fuésemos',
        'fueseis',
        'fuesen',
        'sintiendo',
        'sentido',
        'sentida',
        'sentidos',
        'sentidas',
        'siente',
        'sentid',
        'tengo',
        'tienes',
        'tiene',
        'tenemos',
        'tenéis',
        'tienen',
        'tenga',
        'tengas',
        'tengamos',
        'tengáis',
        'tengan',
        'tendré',
        'tendrás',
        'tendrá',
        'tendremos',
        'tendréis',
        'tendrán',
        'tendría',
        'tendrías',
        'tendríamos',
        'tendríais',
        'tendrían',
        'tenía',
        'tenías',
        'teníamos',
        'teníais',
        'tenían',
        'tuve',
        'tuviste',
        'tuvo',
        'tuvimos',
        'tuvisteis',
        'tuvieron',
        'tuviera',
        'tuvieras',
        'tuviéramos',
        'tuvierais',
        'tuvieran',
        'tuviese',
        'tuvieses',
        'tuviésemos',
        'tuvieseis',
        'tuviesen',
        'teniendo',
        'tenido',
        'tenida',
        'tenidos',
        'tenidas',
        'tened',
    ),
    FR: (
        'au',
        'aux',
        'avec',
        'ce',
        'ces',
        'dans',
        'de',
        'des',
        'du',
        'elle',
        'en',
        'et',
        'eux',
        'il',
        'ils',
        'je',
        'la',
        'le',
        'les',
        'leur',
        'lui',
        'ma',
        'mais',
        'me',
        'même',
        'mes',
        'moi',
        'mon',
        'ne',
        'nos',
        'notre',
        'nous',
        'on',
        'ou',
        'par',
        'pas',
        'pour',
        'qu',
        'que',
        'qui',
        'sa',
        'se',
        'ses',
        'son',
        'sur',
        'ta',
        'te',
        'tes',
        'toi',
        'ton',
        'tu',
        'un',
        'une',
        'vos',
        'votre',
        'vous',
        'c',
        'd',
        'j',
        'l',
        'à',
        'm',
        'n',
        's',
        't',
        'y',
        'été',
        'étée',
        'étées',
        'étés',
        'étant',
        'étante',
        'étants',
        'étantes',
        'suis',
        'es',
        'est',
        'sommes',
        'êtes',
        'sont',
        'serai',
        'seras',
        'sera',
        'serons',
        'serez',
        'seront',
        'serais',
        'serait',
        'serions',
        'seriez',
        'seraient',
        'étais',
        'était',
        'étions',
        'étiez',
        'étaient',
        'fus',
        'fut',
        'fûmes',
        'fûtes',
        'furent',
        'sois',
        'soit',
        'soyons',
        'soyez',
        'soient',
        'fusse',
        'fusses',
        'fût',
        'fussions',
        'fussiez',
        'fussent',
        'ayant',
        'ayante',
        'ayantes',
        'ayants',
        'eu',
        'eue',
        'eues',
        'eus',
        'ai',
        'as',
        'avons',
        'avez',
        'ont',
        'aurai',
        'auras',
        'aura',
        'aurons',
        'aurez',
        'auront',
        'aurais',
        'aurait',
        'aurions',
        'auriez',
        'auraient',
        'avais',
        'avait',
        'avions',
        'aviez',
        'avaient',
        'eut',
        'eûmes',
        'eûtes',
        'eurent',
        'aie',
        'aies',
        'ait',
        'ayons',
        'ayez',
        'aient',
        'eusse',
        'eusses',
        'eût',
        'eussions',
        'eussiez',
        'eussent',
    ),
}


def cleanse(text, language_code: Language = EN) -> List[str]:
    """Filter the stopwords of the language key out."""
    return [w for w in text.lower().split() if w not in WORDS[language_code]]


def language_code_of(language: str) -> Language:
    """Leniently map language as str to language code for indexing into words."""
    lenient = language.strip().lower()
    if lenient == 'german':
        return DE
    if lenient == 'english':
        return EN
    if lenient == 'spanish':
        return ES
    if lenient == 'french':
        return FR

    raise ValueError(f'no stop words available for language {language}')



"""Overlap (Finnish: limitys) assesses sentences from constrained and overlapping vocabularies."""
import argparse
import datetime as dti
import json
import logging
import pathlib
import sys
from typing import Dict, Tuple, Union

import yaml
from gensim import downloader as model_api
from gensim.corpora import Dictionary
from gensim.models import TfidfModel
from gensim.similarities import SparseTermSimilarityMatrix, WordEmbeddingSimilarityIndex

from limitys import ENCODING, log, stop

PathLike = Union[str, pathlib.Path]
Documents = Dict[str, Dict[str, str]]
Verification = Tuple[bool, str]

MODELS = {
    'news': 'word2vec-google-news-300',
    'wiki': 'fasttext-wiki-news-subwords-300',
}
DEFAULT_MODEL = MODELS['news']
DEFAULT_DOCUMENTS_NAME = 'documents.yml'


def load_documents(path: PathLike = DEFAULT_DOCUMENTS_NAME) -> Documents:
    """Load the keyed documents from the file per convention and format (from suffix)."""
    source = pathlib.Path(path)
    log.debug(f'- documents suffix is {source.suffix}')

    if source.suffix.lower() in ('.yaml', '.yml'):
        log.debug('- reading documents as yaml')
        with open(source, 'rt', encoding=ENCODING) as handle:
            return yaml.safe_load(handle)  # type: ignore

    if source.suffix.lower() in ('.json',):
        log.debug('- reading documents as json')
        with open(source, 'rt', encoding=ENCODING) as handle:
            return json.load(handle)  # type: ignore

    return {}


def similarity(options: argparse.Namespace) -> int:
    """Drive the verification."""
    documents = pathlib.Path(options.documents)
    language = options.language
    language_code = stop.language_code_of(language)
    quiet = options.quiet
    if quiet:
        logging.getLogger().setLevel(logging.ERROR)

    start_time = dti.datetime.now(tz=dti.timezone.utc)
    log.info(f'starting similarity analysis of {language} (code {language_code}) documents in ({documents})')
    log.info(f'output channel is {"STDOUT" if options.out_path is sys.stdout else options.out_path}')
    docs = load_documents(documents)
    log.info(f'- loaded {len(docs)} documents from ({documents})')

    if not docs:
        log.error('no documents to analyze')
        return 1

    sentences = {k: stop.cleanse(docs[k], stop.EN) for k in docs}

    dictionary = Dictionary(list(sentences.values()))
    bags_of_words = {k: dictionary.doc2bow(sentences[k]) for k in sentences}

    tfidf = TfidfModel(list(bags_of_words.values()))
    tfidfs = {k: tfidf[bags_of_words[k]] for k in bags_of_words}

    model = model_api.load(MODELS['wiki'])

    termsim_index = WordEmbeddingSimilarityIndex(model)
    termsim_matrix = SparseTermSimilarityMatrix(termsim_index, dictionary, tfidf)

    pbm, row_heads = [], []
    col_heads = [key for key in tfidfs]
    for row, (i, bag) in enumerate(tfidfs.items()):
        pbm.append([])
        row_heads.append(i)
        for col, (j, bah) in enumerate(tfidfs.items()):
            similarity = termsim_matrix.inner_product(bag, bah, normalized=(True, True))
            pbm[row].append(similarity if col > row else None)
            log.debug(f'{i=}, {j=}')
            log.debug(docs[i])
            log.debug(docs[j])
            log.debug('similarity = %.4f' % similarity)
            log.debug('# ' + '- ' * 42)

    matrix_rep = ['# Matrix:', '']
    matrix_rep.append(f'{" ".join(cell.rjust(12) for cell in col_heads[1:])}{" "*12}')
    matrix_rep.append(f'{" ".join(("-"*11).rjust(12) for _ in row_heads[1:])}{" "*12}')
    for rank, row in enumerate(pbm[:-1]):
        upp_tri_mat_row = ' '.join(str('' if cell is None else round(cell, 3)).rjust(12) for cell in row[1:])
        matrix_rep.append(f'{upp_tri_mat_row} | {row_heads[rank] :12s}')
    if options.out_path is sys.stdout:
        log.info('- writing similarity upper triangle matrix to STDOUT')
        print('\n'.join(matrix_rep))
    else:
        out = pathlib.Path(options.out_path)
        log.info(f'- writing similarity upper triangle matrix to {out}')
        with open(out, 'wt', encoding=ENCODING) as handle:
            handle.write('\n'.join(matrix_rep) + '\n')

    end_time = dti.datetime.now(tz=dti.timezone.utc)
    log.info(f'similarity analysis complete after {(end_time - start_time).total_seconds()} seconds')
    return 0



"""CLI interface for overlap (Finnish: limitys) assesses sentences from constrained and overlapping vocabularies."""
import argparse
import pathlib
import sys
from typing import List, Union

import limitys.limitys as api
from limitys import APP_ALIAS, APP_NAME


def parse_request(argv: List[str]) -> Union[int, argparse.Namespace]:
    """DRY."""
    parser = argparse.ArgumentParser(
        prog=APP_ALIAS, description=APP_NAME, formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        '--language',
        '-l',
        dest='language',
        required=False,
        default='english',
        help='language in (english, french, german, spanish) of the documents. Optional\n(default: english)',
    )
    parser.add_argument(
        '--documents',
        '-d',
        dest='documents',
        default='',
        help='file providing the keyed documents. Optional\n(default: positional documents value)',
        required=False,
    )
    parser.add_argument(
        'documents_pos',
        nargs='?',
        default='',
        help='file with format in (json, yaml) providing the keyed documents. Optional\n(default: documents.yml)',
    )
    parser.add_argument(
        '--out-path',
        '-o',
        dest='out_path',
        default=sys.stdout,
        help='output file path for matrix (default: STDOUT)',
    )
    parser.add_argument(
        '--quiet',
        '-q',
        dest='quiet',
        default=False,
        action='store_true',
        help='work as quiet as possible - progress bar only if all well (default: False)',
    )
    parser.add_argument(
        '--verbose',
        '-v',
        dest='verbose',
        default=False,
        action='store_true',
        help='work logging more information along the way (default: False)',
    )
    if not argv:
        parser.print_help()
        return 0

    options = parser.parse_args(argv)

    if options.verbose and options.quiet:
        parser.error('you cannot be quiet and verbose at the same time')

    if not options.documents:
        if options.documents_pos:
            options.documents = options.documents_pos
        else:
            options.documents = 'documents.yml'

    documents = pathlib.Path(options.documents)
    if documents.exists():
        if documents.is_file():
            return options
        parser.error(f'requested documents file at ({documents}) is not a file')

    parser.error(f'requested documents file at ({documents}) does not exist')


def main(argv: Union[List[str], None] = None) -> int:
    """Delegate processing to functional module."""
    argv = sys.argv[1:] if argv is None else argv
    options = parse_request(argv)
    if isinstance(options, int):
        return 0
    return api.similarity(options)
